"""
deployment.py: Contains deployment configurations for each of the
functions.
"""
from __future__ import print_function, division, unicode_literals, \
    absolute_import
from builtins import str

__author__ = u"Jason Wolosonovich <jason@avaland.io>"
__license__ = u"BSD 3 Clause"


ENDPOINT_CONFIGS = {
    u"name": u"the_refinery",
    u"full_name": u"projects/infusionsoft-looker-poc/locations/uc"
                 u"-central1/functions/the_refinery",
    u"region": u"us-central1",
    u"timeout": u"540s",
    u"memory": u"512MB",
    u"runtime": u"python37",
    u"trigger": u"the_refinery",
    u"entry_point": u"run_the_refinery",
    u"source": u"/cloud_functions/services/the_refinery/",
}


CLEARBIT_CONFIGS = {
    u"name": u"the_refinery_clearbit_service",
    u"full_name": u"projects/infusionsoft-looker-poc/locations/uc"
                 u"-central1/functions/the_refinery_clearbit_service",
    u"region": u"us-central1",
    u"timeout": u"540s",
    u"memory": u"512MB",
    u"runtime": u"python37",
    u"trigger": u"the_refinery_clearbit_service",
    u"entry_point": u"run_clearbit_service",
    u"source": u"/cloud_functions/services/clearbit_service/",
}


CRAWLER_CONFIGS = {
    u"name": u"the_refinery_crawler_service",
    u"full_name": u"projects/infusionsoft-looker-poc/locations/uc"
                 u"-central1/functions/the_refinery_crawler_service",
    u"region": u"us-central1",
    u"timeout": u"540s",
    u"memory": u"512MB",
    u"runtime": u"python37",
    u"trigger": u"the_refinery_crawler_service",
    u"entry_point": u"run_crawler_service",
    u"source": u"/cloud_functions/services/crawler_service/",
}


MOBILE_CONFIGS = {
    u"name": u"the_refinery_mobile_friendly_service",
    u"full_name": u"projects/infusionsoft-looker-poc/locations/uc"
                 u"-central1/functions/the_refinery_mobile_friendly_service",
    u"region": u"us-central1",
    u"timeout": u"540s",
    u"memory": u"512MB",
    u"runtime": u"python37",
    u"trigger": u"the_refinery_mobile_friendly_service",
    u"entry_point": u"run_mobile_friendly_service",
    u"source": u"/cloud_functions/services/mobile_friendly_service/",
}


WP_PLUGIN_LOOKUP_CONFIGS = {
    u"name": u"the_refinery_wordpress_plugin_lookup_service",
    u"full_name": u"projects/infusionsoft-looker-poc/locations/uc"
                 u"-central1/functions/the_refinery_wordpress_plugin_lookup_service",
    u"region": u"us-central1",
    u"timeout": u"540s",
    u"memory": u"512MB",
    u"runtime": u"python37",
    u"trigger": u"the_refinery_wordpress_plugin_lookup_service",
    u"entry_point": u"run_wordpress_plugin_lookup_service",
    u"source": u"/cloud_functions/services"
              u"/wordpress_plugin_lookup_service/",
}


EMAIL_PROVIDER_LOOKUP_CONFIGS = {
    u"name": u"the_refinery_email_provider_lookup_service",
    u"full_name": u"projects/infusionsoft-looker-poc/locations/uc"
                 u"-central1/functions/the_refinery_email_provider_lookup_service",
    u"region": u"us-central1",
    u"timeout": u"540s",
    u"memory": u"512MB",
    u"runtime": u"python37",
    u"trigger": u"the_refinery_email_provider_lookup_service",
    u"entry_point": u"run_email_provider_lookup_service",
    u"source": u"/cloud_functions/services"
              u"/email_provider_lookup_service/",
}


"""
schemas.py: Schemas for the BiqQuery tables where the enriched data is
stored.
"""
__author__ = "Jason Wolosonovich <jason@avaland.io>"
__license__ = "BSD 3 clause"

from google.cloud import bigquery as bq

# mobile friendly test
MOBILE_FRIENDLY_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="test_results",
                   field_type="string",
                   mode="required")
]

# clearbit tag history
CLEARBIT_TAGS_HISTORY_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="tag",
                   field_type="string",
                   mode="required")
]

# clearbit tech history
CLEARBIT_TECH_HISTORY_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="tech",
                   field_type="string",
                   mode="required")
]


# clearbit tech history
CLEARBIT_EMAILS_HISTORY_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="email",
                   field_type="string",
                   mode="required")
]


# clearbit tech history
CLEARBIT_PHONES_HISTORY_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="phone",
                   field_type="string",
                   mode="required")
]

# crawler tech history
CRAWLER_WORDPRESS_PLUGIN_HISTORY_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="asset",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="type",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="provider",
                   field_type="string",
                   mode="nullable")
]

# crawler payload
CRAWLER_DOMAIN_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="refinery_person_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_lead_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_contact_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_asset_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_oppty_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_acct_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="app_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="all_links",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="internal_links",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="external_links",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="href_emails",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="href_phones",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="href_socials",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="meta_keywords",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="meta_description",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="tier1_classification",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="tier2_classification",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="tier3_classification",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="classification_likelihood",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="html_string",
                   field_type="string",
                   mode="nullable"),
]

# clearbit person
CLEARBIT_PERSON_SCHEMA = [
    bq.SchemaField(name="refinery_person_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_lead_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_contact_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_asset_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_oppty_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_acct_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="app_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="clearbit_person_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="clearbit_indexed_at",
                   field_type="timestamp",
                   mode="nullable"),
    bq.SchemaField(name="full_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="first_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="last_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="email",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="location",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="time_zone",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="utc_offset",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="city",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="state",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="state_code",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="country",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="country_code",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="latitude",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="longitude",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="bio",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="site",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="avatar",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="employment_domain",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="employment_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="employment_title",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="employment_role",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="employment_seniority",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="facebook_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="github_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="github_avatar",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="github_company",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="github_blog",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="github_followers",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="github_following",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="twitter_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_bio",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_followers",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="twitter_following",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_location",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_site",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_avatar",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="linkedin_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="googleplus_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="gravatar_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="gravatar_url_titles",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="gravatar_urls",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="gravatar_avatar",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="gravatar_avatar_types",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="gravatar_avatar_urls",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="is_email_provider",
                   field_type="boolean",
                   mode="nullable")
]

# clearbit company
CLEARBIT_COMPANY_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="refinery_person_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_lead_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_contact_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_asset_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_oppty_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sfdc_acct_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="app_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_address",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ip_revealed",
                   field_type="boolean",
                   mode="required"),
    bq.SchemaField(name="clearbit_company_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="clearbit_indexed_at",
                   field_type="timestamp",
                   mode="nullable"),
    bq.SchemaField(name="company_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="legal_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="company_domain",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="domain_aliases",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="industry",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="industry_group",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sub_industry",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sector",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sic_code",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="naics_code",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="description",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="year_founded",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="location",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="street_number",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="street_name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="sub_premise",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="city",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="state",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="state_code",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="postal_code",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="country",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="country_code",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="latitude",
                   field_type="float",
                   mode="nullable"),
    bq.SchemaField(name="longitude",
                   field_type="float",
                   mode="nullable"),
    bq.SchemaField(name="time_zone",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="utc_offset",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="company_phone",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="number_of_employees",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="number_of_employees_range",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="fiscal_year_ends",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="market_cap",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="total_raised",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="company_type",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="ticker_symbol",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="tax_ein",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="annual_revenue",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="estimated_annual_revenue",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="company_logo",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="crunchbase_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="alexa_us_rank",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="alexa_global_rank",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="parent_domain",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="facebook_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="facebook_likes",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="linkedin_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_handle",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_avatar",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_bio",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_follower_count",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="twitter_following_count",
                   field_type="integer",
                   mode="nullable"),
    bq.SchemaField(name="twitter_id",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_location",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="twitter_site_url",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="fuzzy_match",
                   field_type="boolean",
                   mode="nullable"),
    bq.SchemaField(name="is_email_provider",
                   field_type="boolean",
                   mode="nullable"),
]


# wp lookup table schema
WP_PLUGIN_LOOKUP_SCHEMA = [
    bq.SchemaField(name="plugin",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="name",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="tag",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="description",
                   field_type="string",
                   mode="nullable")
]


# wp lookup table error schema
WP_PLUGIN_LOOKUP_ERROR_SCHEMA = [
    bq.SchemaField(name="plugin",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required")
]


# email provider lookup table schema
EMAIL_PROVIDER_SCHEMA = [
    bq.SchemaField(name="refinery_company_id",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="domain",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="mx_record",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="email_provider",
                   field_type="string",
                   mode="required")
]


# ip lookup table schema
IP_LOOKUP_SCHEMA = [
    bq.SchemaField(name="refined_at",
                   field_type="timestamp",
                   mode="required"),
    bq.SchemaField(name="refined_date",
                   field_type="date",
                   mode="required"),
    bq.SchemaField(name="ip_address",
                   field_type="string",
                   mode="required"),
    bq.SchemaField(name="reason",
                   field_type="string",
                   mode="nullable"),
    bq.SchemaField(name="source",
                   field_type="string",
                   mode="required")
]


"""
functions.py: Utility functions used throughout the services within the
              Refinery.
"""
from __future__ import print_function, division, unicode_literals, \
    absolute_import

import time
import re
import base64
import json

import requests
from google.cloud import storage, pubsub
from google.cloud import bigquery as bq
import validators
import gcsfs

from .crawler_service import HEADERS

__author__ = "Jason Wolosonovich <jason@avaland.io>"
__license__ = "BSD 3 clause"


def upload_to_gcs(payload=None, project_name=None,bucket_name=None,
                  file_name=None, file_suffix=None):
    """Utility function to upload the json to GCS."""
    # TODO: add `content_type` and `content_encoding` when they become
    #  available to the `metadata` kwarg
    print("upload_to_gcs()")

    try:
        fs = gcsfs.GCSFileSystem(
            project=project_name,
            access="full_control",
            token="cloud",
            # consistency="md5",
            cache_timeout=None,
            secure_serialize=True,
            check_connection=True
        )

        fs.retries = 7
        fs.connect(method="cloud")

        with fs.open(bucket_name + "/" + file_name + file_suffix,
                     "w") as f:
            f.write(json.dumps(payload))
        print("payload: {}".format(payload))
        return "OK, 200"
    except Exception as e:
        err = {
            "error_message": e,
            "status_code": "error uploading to gcs."
        }
        print(err)

    # DEPRECATED
    # i = 0
    # while i < 7:
    #     try:
    #         client = storage.Client()
    #         bucket = client.get_bucket(bucket_name)
    #         blob = bucket.blob(file_name + file_suffix)
    #         blob.upload_from_string(
    #             json.dumps(
    #                 payload
    #             ),
    #             content_type="application/json"
    #         )
    #         print("payload: {}".format(payload))
    #         return "OK, 200"
    #
    #     # TODO: what should we do here?
    #     except Exception as e:
    #         i += 1
    #         err = {
    #             "error_message": e,
    #             "status_code": "error uploading to gcs."
    #         }
    #         print(err)
    #         continue
    return "SEE ERROR LOGS"


def download_from_gcs(bucket_name=None, project_name=None,
                      file_name=None):
    """Utility function to upload the json to GCS."""
    print("download_from_gcs()")
    try:
        fs = gcsfs.GCSFileSystem(
            project=project_name,
            access="full_control",
            token="cloud",
            # consistency="md5",
            cache_timeout=None,
            secure_serialize=True,
            check_connection=True
        )

        fs.retries = 7
        fs.connect(method="cloud")

        # return the payload as a byte string since we don't know what
        # native format it might be in
        payload = fs.cat(bucket_name + "/" + file_name)
        return payload

    except Exception as e:
        err = {
            "error_message": e,
            "status_code": "error uploading to gcs."
        }
        print(err)

    return "SEE ERROR LOGS"


    # # DEPRECATED
    # client = storage.Client()
    # bucket = client.get_bucket(bucket_name)
    # blob = bucket.blob(file_name)
    #
    # try:
    #     payload = json.loads(
    #         blob.download_as_string(
    #             client=client
    #         ),
    #         encoding="utf-8"
    #     )
    #
    #     print("payload: {}".format(payload))
    #     return payload
    #
    # # TODO: what should we do here?
    # except Exception as e:
    #     err = {
    #         "error_message": e,
    #         "status_code": "unknown"
    #     }
    #
    #     print(err)
    return


def callback(future):
    """Callback function to ensure the future has completed."""
    message_id = future.result()
    print("message_id: {} received.".format(message_id))
    return "OK, 200"


def publish_to_endpoint(messages=None, max_bytes=10000000,
                        max_latency=0.05, max_messages=1000,
                        delay_length_in_seconds=None):
    print("publish_to_endpoint()")
    messages_published = 0
    start = time.time()
    # apply the batch settings if specified
    batch_settings = pubsub.types.BatchSettings(
        max_bytes=max_bytes,
        max_latency=max_latency,
        max_messages=max_messages
    )
    pubsub_client = pubsub.PublisherClient(
        batch_settings=batch_settings
    )
    # TODO: find a universal way to encode the message so we don't
    #  have to write different decoding functions in the_refinery to
    #  deal with different publishers; this produces a dict,
    #  but product and marketing will be delivering different payloads
    n = 1
    if isinstance(messages, list):
        pass
    else:
        messages = [messages]
    for message in messages:
        byte_encoded_message = str.encode(
            json.dumps(
                message
            )
        )
        print("topic: {}".format(message["topic"]))
        topic = pubsub_client.topic_path("infusionsoft-looker-poc",
                                         message["topic"])
        future = pubsub_client.publish(topic,
                                       byte_encoded_message)

        # attach the callback
        future.add_done_callback(callback)

        # add some rate-limiting
        if delay_length_in_seconds is not None:
            if n % max_messages == 0:
                time.sleep(delay_length_in_seconds)

        # increment our counter
        n += 1
        messages_published += 1
        print("messages published: {}".format(messages_published))
        print("time elapsed (seconds): {0:.1f}"
              .format(time.time() - start))

    end = time.time()
    print("total publishing time: {0:.1f}".format((end - start) / 60.))
    return "OK, 200"


def decode_event(event):
    """Utility function for decoding the request."""
    # decode the message string
    print("decode_event()")
    request = base64.b64decode(
        event["data"]
    ).decode(
        'utf-8'
    )
    # TODO: see if there is some in-built way in flask to handle all
    #  this in a one-liner
    # convert the message string to a dict
    request = json.loads(request)
    print("event: {}".format(request))
    return request


def get_service_params(request=None):
    """Utility function to find the publisher, services and urls of the
    services to be used."""
    print("get_service_params()")
    # lookup dictionary
    service_lookup = {
        "analytics": {
            "clearbit_service":
                "https://us-central1-infusionsoft-looker-poc"
                ".cloudfunctions.net/the_refinery_clearbit_service",
            "crawler_service":
                "https://us-central1-infusionsoft-looker-poc"
                ".cloudfunctions.net/the_refinery_crawler_service"
        },
        "product": None,
        "marketing": None
    }

    publisher = request["publisher"]
    services = request["services_requested"]
    service_urls = [service_lookup[publisher][service]
                    for service in services]
    return publisher, services, service_urls


def get_valid_url(domain=None):
    """Utility function that produces (when possible) a valid url
    from a domain name."""
    # TODO: add more robust exception handling
    print("get_valid_url()")
    # NOTE: probably not robust to edge-cases
    # this may already be a valid url
    is_valid_url = validators.url(domain)

    if is_valid_url is True:
        print("domain is a valid url")
        url = domain
        # there are edge-cases where this will still pass
        try:
            resp = requests.get(url,
                                headers=HEADERS)
            print("status code: {}".format(resp.status_code))
            # one last check for status
            if resp.status_code != 200:
                print("could not get url: {}".format(url))
                url = None
        except requests.exceptions.HTTPError as e:
            url = None
        except Exception as e:
            url = None

    # validation failed
    else:
        # check to see if it is a valid domain that we can use to
        # construct a url from
        is_valid_domain = validators.domain(domain)
        print("is_valid_domain: {}".format(is_valid_domain))
        # we can't construct a valid url without a valid domain
        if is_valid_domain is not True:
            print("domain is not valid")
            url = None
        else:
            # try to construct a valid url
            if domain.lower().startswith("www."):
                url = "http://" + domain.lower()
            else:
                url = "http://www." + domain.lower()
            print("constructed url: {}".format(url))

            # one more validation check then try to open the url
            is_valid_url = validators.url(url)

            print("is_valid_url: {}".format(is_valid_url))
            # now try to open the url as a final check
            try:
                resp = requests.get(url,
                                    headers=HEADERS)
                print("status code: {}".format(resp.status_code))
                # one last check for status
                if resp.status_code != 200:
                    print("could not get url: {}".format(url))
                    # try https then
                    # try to construct a valid url
                    print("trying https instead")
                    if domain.lower().startswith("www."):
                        url = "https://" + domain.lower()
                    else:
                        url = "https://www." + domain.lower()
                    try:
                        resp = requests.get(url,
                                            headers=HEADERS)
                        print("status code: {}"
                              .format(resp.status_code))
                        if resp.status_code != 200:
                            print("could not get https url: {}"
                                  .format(url))
                            url = None
                    except requests.exceptions.HTTPError as e:
                        url = None
                    except Exception as e:
                        url = None
            except requests.exceptions.HTTPError as e:
                url = None
            except Exception as e:
                url = None
    return url


def is_valid_email(email=None):
    """Validates whether the email provided is syntactically correct."""
    # source: https://stackoverflow.com/questions/201323/how-to-validate-an-email-address-using-a-regular-expression
    # TODO: implement the python equivalent of the state machine
    #  found here: https://github.com/cubiclesoft/ultimate-email
    r = re.compile(
        """(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9]))\.){3}(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9])|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])"""
    )
    try:
        is_valid = False if re.match(r, email) is None else True
    except Exception as e:
        print(e)
        is_valid = False
    return is_valid


def get_domain_from_email(email=None):
    """Returns the domain from the provided email address."""
    r = re.compile(
        """^(.*@)([\w.]+)"""
    )
    try:
        domain = re.match(r, email).group(2)
        print("domain: {}".format(domain))
    except Exception as e:
        print(e)
        domain = None
    return domain


def insert_one_to_bq(dataset=None, table=None, schema=None,
                     payload=None, payload_type=None):
    """Utility function for inserting json rows into the specified
    table."""
    print("insert_one_to_bq()")

    if payload_type == "mobile":
        print("inserting mobile test results.")
        row = [
            (
                payload["refinery_company_id"],
                payload["refined_at"],
                payload["refined_date"],
                payload["domain"],
                payload["url"],
                payload["ip_revealed"],
                payload["fuzzy_match"],
                payload["test_results"]
            )
        ]

    if payload_type == "crawler":
        print("inserting crawler results.")
        row = [
            (
                payload["refinery_company_id"],
                payload["refined_at"],
                payload["refined_date"],
                payload["refinery_person_id"],
                payload["sfdc_lead_id"],
                payload["sfdc_contact_id"],
                payload["sfdc_asset_id"],
                payload["sfdc_oppty_id"],
                payload["sfdc_acct_id"],
                payload["app_name"],
                payload["domain"],
                payload["url"],
                payload["ip_revealed"],
                payload["fuzzy_match"],
                payload["all_links"],
                payload["internal_links"],
                payload["external_links"],
                payload["href_emails"],
                payload["href_phones"],
                payload["href_socials"],
                payload["meta_keywords"],
                payload["meta_description"],
                payload["tier1_classification"],
                payload["tier2_classification"],
                payload["tier3_classification"],
                payload["classification_confidence"],
                payload["html_string"]
            )
        ]

    if payload_type == "person":
        print("inserting clearbit person results.")
        row = [
            (
                payload["refinery_person_id"],
                payload["refined_at"],
                payload["refined_date"],
                payload["refinery_company_id"],
                payload["sfdc_lead_id"],
                payload["sfdc_contact_id"],
                payload["sfdc_asset_id"],
                payload["sfdc_oppty_id"],
                payload["sfdc_acct_id"],
                payload["app_name"],
                payload["domain"],
                payload["url"],
                payload["ip_revealed"],
                payload["clearbit_person_id"],
                payload["clearbit_indexed_at"],
                payload["full_name"],
                payload["first_name"],
                payload["last_name"],
                payload["email"],
                payload["location"],
                payload["time_zone"],
                payload["utc_offset"],
                payload["city"],
                payload["state"],
                payload["state_code"],
                payload["country"],
                payload["country_code"],
                payload["latitude"],
                payload["longitude"],
                payload["bio"],
                payload["site"],
                payload["avatar"],
                payload["employment_domain"],
                payload["employment_name"],
                payload["employment_title"],
                payload["employment_role"],
                payload["employment_seniority"],
                payload["facebook_handle"],
                payload["github_handle"],
                payload["github_avatar"],
                payload["github_company"],
                payload["github_blog"],
                payload["github_followers"],
                payload["github_following"],
                payload["twitter_handle"],
                payload["twitter_id"],
                payload["twitter_bio"],
                payload["twitter_followers"],
                payload["twitter_following"],
                payload["twitter_location"],
                payload["twitter_site"],
                payload["twitter_avatar"],
                payload["linkedin_handle"],
                payload["googleplus_handle"],
                payload["gravatar_handle"],
                payload["gravatar_url_titles"],
                payload["gravatar_urls"],
                payload["gravatar_avatar"],
                payload["gravatar_avatar_types"],
                payload["gravatar_avatar_urls"],
                payload["fuzzy_match"],
                payload["is_email_provider"],

            )
        ]

    if payload_type == "company":
        print("inserting clearbit company results.")
        row = [
            (
                payload["refinery_company_id"],
                payload["refined_at"],
                payload["refined_date"],
                payload["refinery_person_id"],
                payload["sfdc_lead_id"],
                payload["sfdc_contact_id"],
                payload["sfdc_asset_id"],
                payload["sfdc_oppty_id"],
                payload["sfdc_acct_id"],
                payload["app_name"],
                payload["domain"],
                payload["url"],
                payload["ip_address"],
                payload["ip_revealed"],
                payload["clearbit_company_id"],
                payload["clearbit_indexed_at"],
                payload["company_name"],
                payload["legal_name"],
                payload["company_domain"],
                payload["domain_aliases"],
                payload["industry"],
                payload["industry_group"],
                payload["sub_industry"],
                payload["sector"],
                payload["sic_code"],
                payload["naics_code"],
                payload["description"],
                payload["year_founded"],
                payload["location"],
                payload["street_number"],
                payload["street_name"],
                payload["sub_premise"],
                payload["city"],
                payload["state"],
                payload["state_code"],
                payload["postal_code"],
                payload["country"],
                payload["country_code"],
                payload["latitude"],
                payload["longitude"],
                payload["time_zone"],
                payload["utc_offset"],
                payload["company_phone"],
                payload["number_of_employees"],
                payload["number_of_employees_range"],
                payload["fiscal_year_ends"],
                payload["market_cap"],
                payload["total_raised"],
                payload["company_type"],
                payload["ticker_symbol"],
                payload["tax_ein"],
                payload["annual_revenue"],
                payload["estimated_annual_revenue"],
                payload["company_logo"],
                payload["crunchbase_handle"],
                payload["alexa_us_rank"],
                payload["alexa_global_rank"],
                payload["parent_domain"],
                payload["facebook_handle"],
                payload["facebook_likes"],
                payload["linkedin_handle"],
                payload["twitter_handle"],
                payload["twitter_avatar"],
                payload["twitter_bio"],
                payload["twitter_follower_count"],
                payload["twitter_following_count"],
                payload["twitter_id"],
                payload["twitter_location"],
                payload["twitter_site_url"],
                payload["fuzzy_match"],
                payload["is_email_provider"]

            )
        ]

    if payload_type == "wp_lookup_error":
        print("updating wp plugin error table.")
        row = [
            (
                payload["plugin"],
                payload["refined_at"],
                payload["refined_date"]
            )
        ]

    if payload_type == "ip_lookup":
        print("adding row to ip lookup table.")
        row = [
            (
                payload["refined_at"],
                payload["refined_date"],
                payload["ip_address"],
                payload["reason"],
                payload["source"]
            )
        ]

    max_retries = 7
    for n in range(max_retries):
        try:
            # configs for bq
            client = bq.Client()
            # job_config = bq.LoadJobConfig()
            # job_config.schema = schema
            # job_config.source_format = \
            #     bq.job.SourceFormat.NEWLINE_DELIMITED_JSON
            # job_config.write_disposition = \
            #     bq.job.WriteDisposition.WRITE_APPEND

            dataset_ref = client.dataset(dataset)
            dataset = bq.Dataset(dataset_ref)
            table_ref = dataset.table(table)
            table = bq.Table(table_ref,
                             schema=schema)

            print("dataset: {}".format(dataset))
            print("table: {}".format(table))
            print("schema: {}".format(schema))
            print("payload: {}".format(payload))
            print("payload type: {}".format(payload_type))

            # attempt to insert the record
            errors = client.insert_rows(
                table,
                row,
                selected_fields=schema,

            )

            if len(errors) != 0:
                print(errors)
                continue
            success = True
            break
        except Exception as e:
            if n == max_retries - 1:
                print(e)
                success = False
                break
            time.sleep(2 ** n)
            continue
    return "success: {}".format(success)



def insert_many_to_bq(dataset=None, table=None, schema=None,
                      payload=None, payload_type=None):
    """Utility function for inserting json rows into the specified
    table."""
    print("insert_many_to_bq()")

    # configs for bq
    client = bq.Client()
    # job_config = bq.LoadJobConfig()
    # job_config.schema = schema
    # job_config.source_format = \
    #     bq.job.SourceFormat.NEWLINE_DELIMITED_JSON
    # job_config.write_disposition = \
    #     bq.job.WriteDisposition.WRITE_APPEND

    dataset_ref = client.dataset(dataset)
    dataset = bq.Dataset(dataset_ref)
    table_ref = dataset.table(table)
    table = bq.Table(table_ref,
                     schema=schema)

    print("dataset: {}".format(dataset))
    print("table: {}".format(table))
    print("schema: {}".format(schema))
    print("payload: {}".format(payload))
    print("payload type: {}".format(payload_type))

    if payload_type == "crawler_tech":
        keys = payload.keys()
        if "wp_themes" in keys:
            print("inserting themes.")
            for theme in payload["wp_themes"]:
                row = [
                    (
                        payload["refinery_company_id"],
                        payload["refined_at"],
                        payload["refined_date"],
                        payload["domain"],
                        payload["url"],
                        payload["ip_revealed"],
                        payload["fuzzy_match"],
                        theme,
                        "theme",
                        "wordpress"
                    )
                ]

                max_retries = 7
                for n in range(max_retries):
                    try:
                        # attempt to insert the record
                        errors = client.insert_rows(
                            table,
                            row,
                            selected_fields=schema,

                        )

                        if len(errors) != 0:
                            print(errors)
                            continue
                        success = True
                        break
                    except Exception as e:
                        if n == max_retries - 1:
                            print(e)
                            success = False
                            break
                        time.sleep(2 ** n)
                        continue

        if "wp_plugins" in keys:
            print("inserting plugins.")
            for plugin in payload["wp_plugins"]:
                row = [
                    (
                        payload["refinery_company_id"],
                        payload["refined_at"],
                        payload["refined_date"],
                        payload["domain"],
                        payload["url"],
                        payload["ip_revealed"],
                        payload["fuzzy_match"],
                        plugin,
                        "plugin",
                        "wordpress"
                    )
                ]

                max_retries = 7
                for n in range(max_retries):
                    try:
                        # attempt to insert the record
                        errors = client.insert_rows(
                            table,
                            row,
                            selected_fields=schema,

                        )

                        if len(errors) != 0:
                            print(errors)
                            continue
                        success = True
                        break
                    except Exception as e:
                        if n == max_retries - 1:
                            print(e)
                            success = False
                            break
                        time.sleep(2 ** n)
                        continue

    if payload_type == "clearbit_emails":
        for email in payload["emails"]:
            print("inserting emails.")
            row = [
                (
                    payload["refinery_company_id"],
                    payload["refined_at"],
                    payload["refined_date"],
                    payload["domain"],
                    payload["url"],
                    payload["ip_revealed"],
                    payload["fuzzy_match"],
                    email
                )
            ]

            max_retries = 7
            for n in range(max_retries):
                try:
                    # attempt to insert the record
                    errors = client.insert_rows(
                        table,
                        row,
                        selected_fields=schema,

                    )

                    if len(errors) != 0:
                        print(errors)
                        continue
                    success = True
                    break
                except Exception as e:
                    if n == max_retries - 1:
                        print(e)
                        success = False
                        break
                    time.sleep(2 ** n)
                    continue

    if payload_type == "clearbit_phones":
        for phone in payload["phones"]:
            print("inserting phones.")
            row = [
                (
                    payload["refinery_company_id"],
                    payload["refined_at"],
                    payload["refined_date"],
                    payload["domain"],
                    payload["url"],
                    payload["ip_revealed"],
                    payload["fuzzy_match"],
                    phone
                )
            ]
            max_retries = 7
            for n in range(max_retries):
                try:
                    # attempt to insert the record
                    errors = client.insert_rows(
                        table,
                        row,
                        selected_fields=schema,

                    )

                    if len(errors) != 0:
                        print(errors)
                        continue
                    success = True
                    break
                except Exception as e:
                    if n == max_retries - 1:
                        print(e)
                        success = False
                        break
                    time.sleep(2 ** n)
                    continue

    if payload_type == "clearbit_tags":
        for tag in payload["tags"]:
            print("inserting tags.")
            row = [
                (
                    payload["refinery_company_id"],
                    payload["refined_at"],
                    payload["refined_date"],
                    payload["domain"],
                    payload["url"],
                    payload["ip_revealed"],
                    payload["fuzzy_match"],
                    tag
                )
            ]

            max_retries = 7
            for n in range(max_retries):
                try:
                    # attempt to insert the record
                    errors = client.insert_rows(
                        table,
                        row,
                        selected_fields=schema,

                    )

                    if len(errors) != 0:
                        print(errors)
                        continue
                    success = True
                    break
                except Exception as e:
                    if n == max_retries - 1:
                        print(e)
                        success = False
                        break
                    time.sleep(2 ** n)
                    continue

    if payload_type == "clearbit_tech":
        for tech in payload["tech"]:
            print("inserting tech.")
            row = [
                (
                    payload["refinery_company_id"],
                    payload["refined_at"],
                    payload["refined_date"],
                    payload["domain"],
                    payload["url"],
                    payload["ip_revealed"],
                    payload["fuzzy_match"],
                    tech
                )
            ]

            max_retries = 7
            for n in range(max_retries):
                try:
                    # attempt to insert the record
                    errors = client.insert_rows(
                        table,
                        row,
                        selected_fields=schema,

                    )

                    if len(errors) != 0:
                        print(errors)
                        continue
                    break
                except Exception as e:
                    if n == max_retries - 1:
                        print(e)
                        success = False
                        break
                    time.sleep(2 ** n)
                    continue

    if payload_type == "wp_lookup":
        print("updating wp plugin lookup table.")
        for tag in payload["tags"]:
            row = [
                (
                    payload["plugin"],
                    payload["refined_at"],
                    payload["refined_date"],
                    payload["name"],
                    tag,
                    payload["description"]
                )
            ]

            max_retries = 7
            for n in range(max_retries):
                try:
                    # attempt to insert the record
                    errors = client.insert_rows(
                        table,
                        row,
                        selected_fields=schema,

                    )

                    if len(errors) != 0:
                        print(errors)
                        continue
                    success = True
                    break
                except Exception as e:
                    if n == max_retries - 1:
                        print(e)
                        success = False
                        break
                    time.sleep(2 ** n)
                    continue

    if payload_type == "email_provider":
        print("inserting email provider lookup results.")
        for email_provider, mx_record in zip(
                payload["email_providers"], payload["mx_records"]):
            row = [
                (
                    payload["refinery_company_id"],
                    payload["refined_at"],
                    payload["refined_date"],
                    payload["domain"],
                    mx_record,
                    email_provider
                )
            ]

            max_retries = 7
            for n in range(max_retries):
                try:
                    # attempt to insert the record
                    errors = client.insert_rows(
                        table,
                        row,
                        selected_fields=schema,

                    )

                    if len(errors) != 0:
                        print(errors)
                        continue
                    success = True
                    break
                except Exception as e:
                    if n == max_retries - 1:
                        print(e)
                        success = False
                        break
                    time.sleep(2 ** n)
                    continue

    return "success: {}".format(success)



"""
mobile_friendly_service.py: Utility functions used by the service that
                            checks whether a site is mobile-friendly.
"""
from __future__ import print_function, division, unicode_literals, \
    absolute_import

import requests
import time

from .crawler_service import HEADERS

__author__ = "Jason Wolosonovich <jason@avaland.io>"
__license__ = "BSD 3 clause"


# TODO: this needs more thorough investigation/testing to see how/why
# we keep getting 502 errors
def check_mobile_friendly(request=None):
    """Utility function to test whether the supplied domain is mobile
    friendly."""
    print("check_mobile_friendly()")
    print("url: {}".format(request["url"]))
    # pause to avoid being rate-limited
    time.sleep(1)
    json_data = {"url": request["url"]}
    api_url = \
        "https://searchconsole.googleapis.com/v1/urlTestingTools" \
        "/mobileFriendlyTest:run?key" \
        "=AIzaSyCl4UW5RRovdGSf0xyjlwMpJ754n4_CcvM"
    print("sending request for: {}".format(request["url"]))
    success = False
    i = 0
    while success is False and i < 5:
        try:
            resp = requests.post(
                url=api_url,
                headers=HEADERS,
                json=json_data
            )
            resp.raise_for_status()

            print("response: {}".format(resp))
            success = True
            continue
        except requests.exceptions.HTTPError as e:
            print("attempt {} failed.".format(i + 1))
            i += 1
            resp = str(e.response.status_code)
            continue

    if success is True:
        resp_ = {
            "refinery_company_id": request["refinery_company_id"],
            "refined_at": request["refined_at"],
            "refined_date": request["refined_date"],
            "domain": request["domain"],
            "url": request["url"],
            "ip_revealed": request["ip_revealed"],
            "fuzzy_match": request["fuzzy_match"]
        }

    # we never were able to successfully test the url
    if success is False:
        resp_ = None

    else:
        # tests passed successfully
        if resp.json()["testStatus"]["status"] == "COMPLETE":
            resp_["test_results"] = resp.json()[
                "mobileFriendliness"].lower()

        # even errors seem to return a 200 status code?
        elif resp.json()["testStatus"]["status"] != "COMPLETE":
            if "details" in resp.json()["testStatus"].keys():
                resp_["test_results"] = \
                    resp.json()["testStatus"]["status"] + \
                    " : " + \
                    resp.json()["testStatus"]["details"]
            else:
                resp_["test_results"] = \
                    resp.json()["testStatus"]["status"]
    return resp_



"""
constants.py : contains constant values used by the various functions.
"""
from __future__ import print_function, division, unicode_literals, \
    absolute_import

__author__ = "Jason Wolosonovich <jason@avaland.io>"
__license__ = "BSD 3 Clause"


MAX_RETRIES = 4
SLEEP_LENGTH = 10.0



from __future__ import print_function, division, unicode_literals, \
    absolute_import
import re

import dns.resolver
import dns.exception


def get_email_provider(domain=None):
    """Performs the MX lookup for the specified domain."""
    # perform the lookup and coalesce the results
    email_providers = list()
    mx_records = list()

    max_retries = 15
    for n in range(max_retries):
        try:
            print("getting email providers for: {}"
                  .format(domain))

            print("adjusting resolver params")
            # adjust the timeout settings (quicker)
            # https://github.com/rthalley/dnspython/issues/22
            dns.resolver.get_default_resolver().timeout = 10.0
            dns.resolver.get_default_resolver().lifetime = 10.0

            # TODO: verify where the domain is being stored in the request
            for record in dns.resolver.query(domain, "MX"):
                print("attempt: {}".format(n+1))
                # get the current result and parse out the index value
                result = re.sub("\d+\s",
                                "",
                                record.to_text())
                print("mx_record: {}".format(result))

                # append the raw record
                mx_records.append(result)

                # take the last 2 pieces and remove the trailing .
                email_provider = result.split(".")[-3:-1]

                # combine the two parts
                email_provider = ".".join(email_provider)

                print("email_provider: {}".format(email_provider))
                # append to the list
                email_providers.append(email_provider)

            break
        except dns.exception.Timeout as e:
            print("error with {}: {}".format(domain,
                                             e))
            if n == max_retries - 1:
                email_providers = "error"
                mx_records = "error"
                return email_providers, mx_records
            continue
        except dns.exception.FormError as e:
            print("error with {}: {}".format(domain,
                                             e))
            if n == max_retries - 1:
                email_providers = "error"
                mx_records = "error"
                return email_providers, mx_records
            continue
        except dns.exception.UnexpectedEnd as e:
            print("error with {}: {}".format(domain,
                                             e))
            if n == max_retries - 1:
                email_providers = "error"
                mx_records = "error"
                return email_providers, mx_records
            continue
        except dns.exception.TooBig as e:
            print("error with {}: {}".format(domain,
                                             e))
            if n == max_retries - 1:
                email_providers = "error"
                mx_records = "error"
                return email_providers, mx_records
            continue
        except dns.exception.SyntaxError as e:
            print("error with {}: {}".format(domain,
                                             e))
            if n == max_retries - 1:
                email_providers = "error"
                mx_records = "error"
                return email_providers, mx_records
            continue
        except dns.exception.DNSException as e:
            print("error with {}: {}".format(domain,
                                             e))
            if n == max_retries - 1:
                email_providers = "error"
                mx_records = "error"
                return email_providers, mx_records
            continue

    return email_providers, mx_records



"""
RFTK: A python utility package for the Infusionsoft Refinery.
"""
from __future__ import print_function, division, unicode_literals, \
    absolute_import

from .classes import MetadataMixin

from .functions import get_service_params, get_valid_url, \
    upload_to_gcs, publish_to_endpoint, decode_event, \
    download_from_gcs, insert_one_to_bq, insert_many_to_bq, \
    is_valid_email, get_domain_from_email

from .schemas import MOBILE_FRIENDLY_SCHEMA, \
    CRAWLER_WORDPRESS_PLUGIN_HISTORY_SCHEMA, \
    CLEARBIT_TAGS_HISTORY_SCHEMA, \
    CLEARBIT_TECH_HISTORY_SCHEMA, CRAWLER_DOMAIN_SCHEMA, \
    CLEARBIT_PERSON_SCHEMA, CLEARBIT_EMAILS_HISTORY_SCHEMA, \
    CLEARBIT_PHONES_HISTORY_SCHEMA, CLEARBIT_COMPANY_SCHEMA, \
    WP_PLUGIN_LOOKUP_SCHEMA, WP_PLUGIN_LOOKUP_ERROR_SCHEMA, \
    EMAIL_PROVIDER_SCHEMA, IP_LOOKUP_SCHEMA

from .constants import MAX_RETRIES, SLEEP_LENGTH

from .crawler_service import HEADERS

from .deployment import CLEARBIT_CONFIGS, CRAWLER_CONFIGS, \
    ENDPOINT_CONFIGS, MOBILE_CONFIGS, WP_PLUGIN_LOOKUP_CONFIGS

from .wordpress_lookup_service import get_wp_plugin_info_online

from .email_provider_lookup_service import get_email_provider

__version__ = "0.6.21"
__author__ = "Jason Wolosonovich <jason@avaland.io>"
__license__ = "BSD 3 clause"

# NOTE: should follow the same versioning cadence as the_refinery
__all__ = [
    "get_valid_url",
    "is_valid_email",
    "get_domain_from_email",
    "get_service_params",
    "upload_to_gcs",
    "MOBILE_FRIENDLY_SCHEMA",
    "CRAWLER_WORDPRESS_PLUGIN_HISTORY_SCHEMA",
    "CLEARBIT_TECH_HISTORY_SCHEMA",
    "CLEARBIT_TAGS_HISTORY_SCHEMA",
    "CLEARBIT_PHONES_HISTORY_SCHEMA",
    "CLEARBIT_EMAILS_HISTORY_SCHEMA",
    "CRAWLER_DOMAIN_SCHEMA",
    "CLEARBIT_COMPANY_SCHEMA",
    "CLEARBIT_PERSON_SCHEMA",
    "WP_PLUGIN_LOOKUP_SCHEMA",
    "WP_PLUGIN_LOOKUP_ERROR_SCHEMA",
    "EMAIL_PROVIDER_SCHEMA",
    "IP_LOOKUP_SCHEMA",
    "MetadataMixin",
    "MAX_RETRIES",
    "SLEEP_LENGTH",
    "HEADERS",
    "CLEARBIT_CONFIGS",
    "CRAWLER_CONFIGS",
    "MOBILE_CONFIGS",
    "ENDPOINT_CONFIGS",
    "WP_PLUGIN_LOOKUP_CONFIGS",
    "get_wp_plugin_info_online",
    "get_email_provider",
    "insert_one_to_bq",
    "insert_many_to_bq",
    "publish_to_endpoint"
]
